# COLOR
RED = "\x1b[31m"
DEFAULT = "\x1b[39m"
GREEN = "\x1b[32m"
YELLOW = "\x1b[33m"
# FOLDER
# ROOT_PATH = os.path.dirname(os.path.realpath(__file__)) + '/'
DIST = 'dist'
TEST_FOLDER = 'TEST/'
TEMPLATEFOLDER = 'template/'
OT_FOLDER = '.oiTerminal'
# FILE
# CONFIG_FILE = ROOT_PATH + 'config.json'
STATE_FILE = 'state.json'
TEST_PY = 'test.py'
SUBMIT_PY = 'submit.py'
# LANG_CONFIG_FILE = ROOT_PATH + 'lang.json'
OT_LOG = 'log/oiTerminal.log'
USER_CONFIG_FILE = '_userConfig.json'
COOKIES_FILE = '_cookies.json'
ACCT_CONFIG_FILE = '_acctConfig.json'  # TODO split config json
TEMP_CONFIG_FILE = '_tempConfig.json'
# LOG
APP_NAME = 'oiTerminal'

IN_PREFIX = 'in.'
OUT_PREFIX = 'out.'

# CONFIG
FETCH_RESULT_INTERVAL = 2
CIPHER_KEY = 'oiTerminal'
